

'''
	python3 STATUS.py "FS/DIRECTORY/EQUALIZE/STATUS_1/STATUS_1.py"
'''

import BOTANY.FS.DIRECTORY.DEALLOCATE as DEALLOCATE	
import BOTANY.FS.DIRECTORY.EQUALIZE as EQUALIZE
import BOTANY.PY.VOW as VOW

import json

def PATH (DIRECTORY):
	import pathlib
	FIELD = pathlib.Path (__file__).parent.resolve ()

	from os.path import dirname, join, normpath
	import sys
	return normpath (join (FIELD, DIRECTORY))

def CHECK_1 ():
	START = PATH ("START")
	END = PATH ("END")

	print ("START:", START)
	print ("END:", END)

	VOW.EQUAL (1, 1, lambda VALUES : print (VALUES))

	try:
		DEALLOCATE.DIRECTORY (END)
	except Exception as E:
		print (E)

	OUTPUT = EQUALIZE.MULTIPLE ({
		"DIRECTORY 1": START,
		"DIRECTORY 2": END,
		
		"DIRECTORIES": [
			"1",
			"2",
			"3"
		],
		
		"START": "yes"	
	})
	
	VOW.EQUAL (
		OUTPUT,
		[
			{
				"EQ CHECK": {
					"1": {},
					"2": {}
				},
				"SIZES": {
					"FROM": 10,
					"TO": 10
				}
			},
			{
				"EQ CHECK": {
					"1": {},
					"2": {}
				},
				"SIZES": {
					"FROM": 7,
					"TO": 7
				}
			},
			{
				"EQ CHECK": {
					"1": {},
					"2": {}
				},
				"SIZES": {
					"FROM": 4,
					"TO": 4
				}
			}
		],
		lambda VALUES : print (VALUES)
	)
	
	#print ("OUTPUT:", json.dumps (OUTPUT, indent = 4))
	
	DEALLOCATE.DIRECTORY (END)

	return;
	
	
CHECKS = {
	"CHECK 1": CHECK_1
}





'''
CAUTION: HAS ACCESS TO SHELL
'''

'''
import BOTANY.FS.DIRECTORY.SIZE as SIZE
SIZE.FIND ({
	"DIRECTORY PATH": ""
})
'''

'''
https://stackoverflow.com/questions/1392413/calculating-a-directorys-size-using-python
'''

import subprocess
from os.path import dirname, join, normpath

from pathlib import Path

def FIND (SHARES):
	DIRECTORY_PATH = SHARES ["DIRECTORY PATH"]
	return sum (
		p.stat().st_size for p in Path (DIRECTORY_PATH).rglob ('*')
	)


def DU (SHARES):
	DIRECTORY_PATH = SHARES ["DIRECTORY PATH"]

	import subprocess
	from os.path import dirname, join, normpath
	
	SIZE = subprocess.run (
		f"du -sh '{ DIRECTORY_PATH }'",
		
		shell = True, 
		check = True,
		
		capture_output = True, 
		text = True,
		cwd = normpath (join (dirname (__file__)))
	).stdout.strip ("\n")
	
	return SIZE







import botanical.paths.directory.check_equality as check_equality

def PATH (DIRECTORY):
	import pathlib
	FIELD = pathlib.Path (__file__).parent.resolve ()

	from os.path import dirname, join, normpath
	import sys
	return normpath (join (FIELD, DIRECTORY))

def CHECK_1 ():
	DIRECTORY_1 = PATH ("DIRECTORIES/EQ_1")
	DIRECTORY_2 = PATH ("DIRECTORIES/EQ_2")

	REPORT = check_equality.start (
		DIRECTORY_1,
		DIRECTORY_2
	)	
	
	print (REPORT)
		
	assert (
		{
			'1': {
				'1.HTML': 'f', 
				'1/1.HTML': 'f'
			}, 
			'2': {
				'2': 'd', 
				'2.HTML': 'f', 
				'1/2.HTML': 'f'
			}
		} ==
		REPORT
	)
	
checks = {
	"EQ check with differences": CHECK_1
}






import botanical.paths.directory.check_equality as check_equality

def PATH (DIRECTORY):
	import pathlib
	FIELD = pathlib.Path (__file__).parent.resolve ()

	from os.path import dirname, join, normpath
	import sys
	return normpath (join (FIELD, DIRECTORY))

def CHECK_1 ():
	DIRECTORY_1 = PATH ("DIRECTORIES/EQ_1")
	DIRECTORY_2 = PATH ("DIRECTORIES/EQ_2")

	report = check_equality.start (
		PATH ("DIRECTORIES/EQ_1"),
		 PATH ("DIRECTORIES/EQ_2")
	)	
	assert (
		{'1': {}, '2': {}} ==
		report
	)
	
checks = {
	"EQ check without differences": CHECK_1
}




'''
	PLAN:

import BOTANY.FS.DIRECTORY.EQUALIZE as EQUALIZE

EQUALIZE.MULTIPLE ({
	"DIRECTORY 1": "",
	"DIRECTORY 2": "",
	
	"DIRECTORIES": [
			
	],
	
	"SIZE CHECK": "du",
	
	"START": "no"	
})
'''

import BOTANY.FS.DIRECTORY.RSYNC as RSYNC
import BOTANY.FS.DIRECTORY.CHECK_EQ as CHECK_EQ
import BOTANY.FS.DIRECTORY.SIZE as SIZE

import BOTANY.PY.VOW as VOW

from os.path import dirname, join, normpath

def MULTIPLE (SHARES):
	DIRECTORY_1 = SHARES ["DIRECTORY 1"]
	DIRECTORY_2 = SHARES ["DIRECTORY 2"]
	
	DIRECTORIES = SHARES ["DIRECTORIES"]
	
	if ("START" in SHARES and SHARES ["START"] == "yes"):
		START = "yes"
	else:
		START = "no"
		
	if ("SIZE CHECK" in SHARES and SHARES ["SIZE CHECK"] == "du"):
		SIZE_CHECK = "du"
	else:
		SIZE_CHECK = "py"
	
	OUTPUT = []
	
	for DIRECTORY in DIRECTORIES:	
		FROM = normpath (join (DIRECTORY_1, DIRECTORY))
		TO = normpath (join (DIRECTORY_2, DIRECTORY))
	
		STRING = RSYNC.PROCESS ({
			"FROM": FROM,
			"TO": TO,
			
			"START": START
		})
		
		if (START == "no"):
			print ("STRING:", STRING)
			
		REPORT = CHECK_EQ.START (
			FROM,
			TO
		)	
		VOW.EQUAL (
			REPORT,
			{'1': {}, '2': {}},
			lambda PARAMS : print (PARAMS)
		)
		
		if (SIZE_CHECK == "du"):
			SIZE_FROM = SIZE.DU ({
				"DIRECTORY PATH": FROM
			})
			SIZE_TO = SIZE.DU ({
				"DIRECTORY PATH": TO
			})
			
		else: 
			SIZE_FROM = SIZE.FIND ({
				"DIRECTORY PATH": FROM
			})
			SIZE_TO = SIZE.FIND ({
				"DIRECTORY PATH": TO
			})
		
		OUTPUT.append ({
			"EQ CHECK": REPORT,
			"SIZES": {
				"FROM": SIZE_FROM,
				"TO": SIZE_TO
			}
		})
		
	return OUTPUT




'''
	import BOTANY.FS.DIRECTORY.RSYNC as RSYNC
	PROCESS = RSYNC.PROCESS ({
		"FROM": 
		"TO": 
		
		#
		#	if "no", return the process script, but don't run it
		#
		#	if "yes", start rsync
		#
		"START": "no"
	})
'''

import BOTANY.PROCESS.STARTER as PROCESS_STARTER

RSYNC_LOCATION = "rsync"

def PROCESS (SHARES):
	if ("START" in SHARES and SHARES ["START"] == "yes"):
		START = "yes"
	else:
		START = "no"

	assert ("FROM" in SHARES)
	assert ("TO" in SHARES)

	FROM_DIRECTORY = SHARES ["FROM"]
	TO_DIRECTORY = SHARES ["TO"]

	'''
		--archive, -a            archive mode is -rlptgoD (no -A,-X,-U,-N,-H)
		--verbose, -v            increase verbosity
		
		--mkpath				make directories necessary
	'''
	ACTIVITY = f'{ RSYNC_LOCATION } --mkpath --progress --delete -av "{ FROM_DIRECTORY }/" "{ TO_DIRECTORY }"';
	
	if (START != "yes"):
		return ACTIVITY
	
	PROCESS_STARTER.START (ACTIVITY)

	return;